package junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.data.mongodb.repository.MongoRepository;

import sample.vo.ChildCustomer;
import sample.vo.Customer;

import com.poscoict.glueframework.biz.control.GlueBizControlConstants;
import com.poscoict.glueframework.biz.control.GlueBizProvider;
import com.poscoict.glueframework.context.GlueContext;
import com.poscoict.glueframework.context.GlueDefaultContext;
import com.poscoict.glueframework.context.GlueStaticContext;

public class MongoDBActivityTest
{

    private GlueContext ctx;
    private String serviceName;
    Customer customer;

    @BeforeClass
    public static void onlyOnceStart()
    {
        TestInitializer.initialize();
    }

    @Before
    public void init()
    {
        serviceName = "sample-service";
        customer = new Customer( "yujin", "ho" );

        // data를 지움.
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.deleteAll();
    }

    @Test
    public void test_GlueMongoSaveOne()
    {
        // GlueMongoSave Activity Test
        // 1개 Document 객체를 이용해 추가
        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, serviceName );
        ctx.put( "saveOne", "event" );
        ctx.put( "data", customer );
        GlueBizProvider.getController().doAction( ctx );

        Object saveResult = ctx.get( "customerRepository_result" );
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        List findResult = repository.findAll();
        assertTrue( findResult.size() == 1 );
        assertEquals( findResult.get( 0 ).getClass(), Customer.class );
        assertEquals( findResult.get( 0 ).getClass(), saveResult.getClass() );
    }

    @Test
    public void test_GlueMongoSaveEntity()
    {
        // GlueMongoSave Activity Test
        // 1개 Document 생성 정보를 이용해 추가
        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "saveEntity", "event" );
        ctx.put( "firstName", customer.getFirstName() );
        ctx.put( "lastName", customer.getLastName() );
        GlueBizProvider.getController().doAction( ctx );

        Object saveResult = ctx.get( "customerRepository_result" );
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        List findResult = repository.findAll();
        assertTrue( findResult.size() == 1 );
        assertEquals( findResult.get( 0 ).getClass(), Customer.class );
        assertEquals( findResult.get( 0 ).getClass(), saveResult.getClass() );
    }

    @Test
    public void test_GlueMongoSaveList()
    {
        // GlueMongoSave Activity Test
        // n개 Document 객체를 이용해 추가
        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "saveList", "1" );
        List<Customer> cList = new ArrayList<Customer>();
        cList.add( new Customer( "yujin B", "ha" ) );
        cList.add( new ChildCustomer( "yujin C", "hwang", "a@abc.com" ) );
        ctx.put( "data", cList );
        GlueBizProvider.getController().doAction( ctx );

        System.out.println(ctx);
        Object saveResult = ctx.get( "customerRepository_result" );
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        List findResult = repository.findAll();
        assertTrue( findResult.size() == 2 );
        assertEquals( findResult.get( 0 ).getClass(), Customer.class );
        assertEquals( findResult.get( 1 ).getClass(), ChildCustomer.class );
        assertEquals( findResult.getClass(), saveResult.getClass() );
    }

    @Test
    public void test_GlueMongoDeleteAll()
    {
        // GlueMongoDelete Activity Test
        // 전체 삭제
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin", "ho" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 2 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "deleteAll", "event" );
        GlueBizProvider.getController().doAction( ctx );

        findResult = repository.findAll();
        assertTrue( findResult.size() == 0 );
    }

    @Test
    public void test_GlueMongoDeleteById()
    {
        // GlueMongoDelete Activity Test
        // ID로 의해 1개 삭제
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin", "ho" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 2 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "deleteEntity", "event" );
        ctx.put( "data", findResult.get( 0 ).getId() );
        System.out.println("---------------------------");
        System.out.println(findResult.get( 0 ).getId());
        System.out.println("---------------------------");
        GlueBizProvider.getController().doAction( ctx );

        findResult = repository.findAll();
        assertTrue( findResult.size() == 1 );
    }

    @Test
    public void test_GlueMongoDeleteByEntity()
    {
        // GlueMongoDelete Activity Test
        // T(entity)로 의해 1개 삭제
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin", "ho" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 2 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "deleteEntity", "event" );
        ctx.put( "data", findResult.get( 0 ) );
        GlueBizProvider.getController().doAction( ctx );

        findResult = repository.findAll();
        assertTrue( findResult.size() == 1 );
    }

    @Test
    public void test_GlueMongoDeleteByEntityList()
    {
        // GlueMongoDelete Activity Test
        // List<T(entity)>로 의해 n개 삭제
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin2", "ho2" ) );
        repository.save( new Customer( "yujin3", "ho3" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 3 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "deleteEntity", "event" );
        findResult.remove( 1 );
        ctx.put( "data", findResult );
        GlueBizProvider.getController().doAction( ctx );

        findResult = repository.findAll();
        assertTrue( findResult.size() == 1 );
    }

    @Test
    public void test_GlueMongoFindAll()
    {
        // GlueMongoFind Activity Test
        // 전체 조회
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin2", "ho2" ) );
        repository.save( new Customer( "yujin3", "ho3" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 3 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "findAll", "event" );
        GlueBizProvider.getController().doAction( ctx );

        List<Customer> result = (List<Customer>) ctx.get( "customerRepository" );
        assertTrue( findResult.size() == result.size() );
    }

    @Test
    public void test_GlueMongoFindByID()
    {
        // GlueMongoFind Activity Test
        // ID에 의해 1개 조회
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin2", "ho2" ) );
        repository.save( new Customer( "yujin3", "ho3" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 3 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "find", "event" );
        ctx.put( "data", findResult.get( 0 ).getId() );
        GlueBizProvider.getController().doAction( ctx );

        Object result = ctx.get( "customerRepository" );
        assertTrue( findResult.get( 0 ).getClass() == result.getClass() );
        assertTrue( findResult.get( 0 ).getId().equals( ( (Customer) result ).getId() ) );
    }

    @Test
    public void test_GlueMongoFindByIDs()
    {
        // GlueMongoFind Activity Test
        // ID에 의해 1개 조회
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin2", "ho2" ) );
        repository.save( new Customer( "yujin3", "ho3" ) );
        List<Customer> findResult = repository.findAll();
        List<Serializable> ids = new ArrayList<Serializable>();
        for ( Customer customer : findResult )
        {
            ids.add( customer.getId() );
        }
        ids.remove( 1 );
        assertTrue( findResult.size() == 3 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "find", "event" );
        ctx.put( "data", ids );
        GlueBizProvider.getController().doAction( ctx );

        List<Customer> result = (List<Customer>) ctx.get( "customerRepository" );
        assertTrue( findResult.get( 0 ).getClass() == result.get( 0 ).getClass() );
        assertTrue( ids.get( 0 ).equals( result.get( 0 ).getId() ) );
        assertTrue( ids.get( 1 ).equals( result.get( 1 ).getId() ) );
    }

    @Test
    public void test_GlueMongoFindAllSorted()
    {
        // GlueMongoFind Activity Test
        // ID에 의해 1개 조회
        MongoRepository repository = GlueStaticContext.getBeanFactory().getBeanObject( "customerRepository", MongoRepository.class );
        repository.save( customer );
        repository.save( new Customer( "yujin 3", "ho2" ) );
        repository.save( new Customer( "yujin 2", "ho3" ) );
        List<Customer> findResult = repository.findAll();
        assertTrue( findResult.size() == 3 );

        ctx = new GlueDefaultContext();
        ctx.put( GlueBizControlConstants.SERVICE_NAME, "sample-service" );
        ctx.put( "findAllSort", "event" ); // firstName=asc|lastName=desc
        GlueBizProvider.getController().doAction( ctx );

        List<Customer> result = (List<Customer>) ctx.get( "customerRepository" );
        assertEquals( result.get( 0 ).getFirstName(), "yujin" );
        assertEquals( result.get( 1 ).getFirstName(), "yujin 2" );
    }

}
